/*
** Module   : EditorImpl.java
** Abstract : ChUI editor implementation
**
** Copyright (c) 2011-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20111121 Extracted CHUI-specific part from Editor.
** 002 CA  20140805 Do not allow ChUI impl classes access to APIs outside of ChuiOutputManager.
** 003 HC  20140911 GUI coordinate support - data types refactoring.
** 004 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 005 GES 20150103 Changes to match the move of some parent class state into the config class.
** 006 IAS 20150326 Fixed typo
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.*;
import com.goldencode.util.*;

/**
 * CHUI-specific portion of EDITOR component.
 */
@SuppressWarnings("unchecked")
public class EditorImpl
extends Editor<ChuiOutputManager>
{
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public EditorImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (EditorConfig) cfg);
   }
   
   /**
    * Sets position of terminal's cursor unconditionally (even if this editor
    * is not in focus).
    */
   public void drawCaretUnconditionally()
   {
      if (!isEnabled())
      {
         return;
      }

      Point origin = new Point(screenLocation());
      
      int box = (config.box) ? 1 : 0;
      
      origin = origin.offset(config.cursorX + box, config.cursorY + box);
      
      screen().at(origin.x, origin.y);
   }

   /**
    * Actual drawing routine.
    *
    * @param   color
    *          Color for drawing.
    */
   public void draw(Color color)
   {
      boolean uninit = (getState() == ScreenBuffer.UNINITIALIZED &&
                        (content == null || content.length() == 0));
      Point start = screenLocation();
      
      // in chui char coordinates are always whole numbers
      int width  = (int) width();
      int height = (int) height();
      
      ChuiOutputManager screen = screen();
      
      if (config.box)
      {
         Color clr  = color.withoutUnderline();
         Color rclr = clr.withReverse();

         screen.box(start, dimension(), clr);

         if (config.scrollbarVertical && height >= 5)
         {
            int scrollbar_offset = (int) calcVpos(height - 4) + 1;

            for (int i = 0; i < (height - 2); i++)
            {
               screen.at(start.offset(width - 1, i + 1))
                     .append(' ', (i == scrollbar_offset) ? clr : rclr);            
            }
         }
         
         if (config.scrollbarHorizontal && width >= 5)
         {
            int scrollbar_offset = (int) calcHpos(width - 4) + 1;

            for (int i = 0; i < (width - 2); i++)
            {
               screen.at(start.offset(i + 1, height - 1))
                     .append(' ', (i == scrollbar_offset) ? clr : rclr);
            }
         }
         
         start = start.offset(1, 1);
         width -= 2;
         height -= 2;
      }
      
      String space = StringHelper.repeatChar(' ', (int) width);

      int line = config.scrollY;
      
      for (int i = 0; i < height; i++, line++)
      {
         String text = prepareTextRow(uninit, width, space, line);
         
         screen.at(start.offset(0, i)).append(text, color);
      }

      if (widgetFocused && isEnabled())
      {
         drawCaret();
         setInsertMode(insert);
      }
   }
}
